#!/usr/bin/env python

import os
import re
import subprocess
import tempfile
import textwrap
import unittest


class KconfgenBaseTestCase(unittest.TestCase):
    @classmethod
    def setUpClass(self):
        self.args = dict()
        self.functions = {
            "in": self.assertIn,
            "not in": self.assertNotIn,
            "equal": self.assertEqual,
            "not equal": self.assertNotEqual,
        }

        try:
            regex_func = self.assertRegex
        except AttributeError:
            # Python 2 fallback
            regex_func = self.assertRegexpMatches
        finally:
            self.functions["regex"] = lambda instance, s, expr: regex_func(
                instance, expr, s
            )  # reverse args order

    def setUp(self):
        with tempfile.NamedTemporaryFile(prefix="test_kconfgen_", delete=False) as f:
            self.output_file = f.name
            self.addCleanup(os.remove, self.output_file)

    def invoke_kconfgen(self, args):
        call_args = ["python", "-m", "kconfgen"]

        for (k, v) in args.items():
            if k != "output":
                if isinstance(v, type("")):  # easy Python 2/3 compatible str/unicode
                    call_args += ["--{}".format(k), v]
                else:
                    for i in v:
                        call_args += ["--{}".format(k), i]
        call_args += [
            "--output",
            args["output"],
            self.output_file,
        ]  # these arguments belong together

        subprocess.check_call(call_args)

    def invoke_and_test(self, in_text, out_text, test="in"):
        """
        Main utility function for testing kconfgen:

        - Runs kconfgen via invoke_kconfgen(), using output method pre-set in test class setup
        - in_text is the Kconfig file input content
        - out_text is some expected output from kconfgen
        - 'test' can be any function key from self.functions dict (see above). Default is 'in' to test if
          out_text is a substring of the full kconfgen output.
        """

        with tempfile.NamedTemporaryFile(
            mode="w+", prefix="test_kconfgen_", delete=False
        ) as f:
            self.addCleanup(os.remove, f.name)
            f.write(textwrap.dedent(in_text))

        self.args["kconfig"] = f.name

        self.invoke_kconfgen(self.args)

        with open(self.output_file) as f_result:
            result = f_result.read()

        try:
            out_text = textwrap.dedent(out_text)
        except TypeError:
            pass  # probably a regex

        self.functions[test](self, out_text, result)
        return result


class CmakeTestCase(KconfgenBaseTestCase):
    @classmethod
    def setUpClass(self):
        super(CmakeTestCase, self).setUpClass()
        self.args.update({"output": "cmake"})

    def testStringEscape(self):
        self.invoke_and_test(
            """
        config PASSWORD
            string "password"
            default "\\\\~!@#$%^&*()\\\""
        """,
            'set(CONFIG_PASSWORD "\\\\~!@#$%^&*()\\"")',
        )

    def testHexPrefix(self):
        self.invoke_and_test(HEXPREFIX_KCONFIG, 'set(CONFIG_HEX_NOPREFIX "0x33")')
        self.invoke_and_test(HEXPREFIX_KCONFIG, 'set(CONFIG_HEX_PREFIX "0x77")')


class JsonTestCase(KconfgenBaseTestCase):
    @classmethod
    def setUpClass(self):
        super(JsonTestCase, self).setUpClass()
        self.args.update({"output": "json"})

    def testStringEscape(self):
        self.invoke_and_test(
            """
        config PASSWORD
            string "password"
            default "\\\\~!@#$%^&*()\\\""
        """,
            '"PASSWORD": "\\\\~!@#$%^&*()\\""',
        )

    def testHexPrefix(self):
        # hex values come out as integers in JSON, due to no hex type
        self.invoke_and_test(HEXPREFIX_KCONFIG, '"HEX_NOPREFIX": %d' % 0x33)
        self.invoke_and_test(HEXPREFIX_KCONFIG, '"HEX_PREFIX": %d' % 0x77)


class JsonMenuTestCase(KconfgenBaseTestCase):
    @classmethod
    def setUpClass(self):
        super(JsonMenuTestCase, self).setUpClass()
        self.args.update({"output": "json_menus"})

    def testMultipleRanges(self):
        self.invoke_and_test(
            """
        config IDF_TARGET
            string "IDF target"
            default "esp32"

        config SOME_SETTING
            int "setting for the chip"
            range 0 100 if IDF_TARGET="esp32s0"
            range 0 10 if IDF_TARGET="esp32"
            range -10 1 if IDF_TARGET="esp32s2"
        """,
            re.compile(r'"range":\s+\[\s+0,\s+10\s+\]'),
            "regex",
        )

    def testHexRanges(self):
        self.invoke_and_test(
            """
        config SOME_SETTING
            hex "setting for the chip"
            range 0x0 0xaf if UNDEFINED
            range 0x10 0xaf
        """,
            r'"range":\s+\[\s+16,\s+175\s+\]',
            "regex",
        )


class ConfigTestCase(KconfgenBaseTestCase):
    @classmethod
    def setUpClass(self):
        super(ConfigTestCase, self).setUpClass()
        self.args.update({"output": "config"})
        self.input = """
        config TEST
            bool "test"
            default "n"
        """

    def setUp(self):
        super(ConfigTestCase, self).setUp()
        with tempfile.NamedTemporaryFile(
            mode="w+", prefix="test_kconfgen_", delete=False
        ) as f:
            self.addCleanup(os.remove, f.name)
            self.args.update(
                {"config": f.name}
            )  # this is input in contrast with {'output': 'config'}
            f.write(
                textwrap.dedent(
                    """
            CONFIG_TEST=y
            CONFIG_UNKNOWN=y
            """
                )
            )

    def testKeepSavedOption(self):
        self.invoke_and_test(self.input, "CONFIG_TEST=y")

    def testDiscardUnknownOption(self):
        self.invoke_and_test(self.input, "CONFIG_UNKNOWN", "not in")


class RenameConfigTestCase(KconfgenBaseTestCase):
    @classmethod
    def setUpClass(self):
        super(RenameConfigTestCase, self).setUpClass()
        # `args` attribute is a dictionary containing the parameters to pass to `kconfgen.py`.
        # Specify the name of the output file, this will generate the argument `--output config`.
        self.args.update({"output": "config"})
        # Setup the KConfig file content in the `input` attribute.
        # Let's define an option that is enabled by default, this is very important.
        # Indeed, as we explicitly disables it by its former name below, rename will be considered as functional
        # if the new name, `(CONFIG_)RENAMED_OPTION` is also disabled in the final configuration file.
        self.input = """
        config RENAMED_OPTION
            bool "Renamed option"
            default y
        """

    def setUp(self):
        super(RenameConfigTestCase, self).setUp()
        # Setup the actual test. What we want to do is to have a configuration file containing which
        # option should be enabled or not, this is the equivalent of the `sdkconfig` that we can find
        # in the examples.
        with tempfile.NamedTemporaryFile(
            mode="w+", prefix="test_kconfgen_", delete=False
        ) as f:
            self.addCleanup(os.remove, f.name)
            # The current file name will be given to `kconfgen.py` after `--config` argument.
            self.args.update({"config": f.name})
            # Specify the content of that configuration file, in our case, we want to explicitely
            # have an option, which needs to be renamed, disabled/not set.
            f.write(
                textwrap.dedent(
                    """
            # CONFIG_NAMED_OPTION is not set
            """
                )
            )
        # The configuration file is ready, we need to prepare a `rename` configuration file which will
        # provide the new name for `CONFIG_NAMED_OPTION` we defined above
        with tempfile.NamedTemporaryFile(
            mode="w+", prefix="test_kconfgen_", delete=False
        ) as f:
            self.addCleanup(os.remove, f.name)
            # Same as above, the following entry will result in the generation of `--sdkconfig-rename`
            # parameter followed by the current temporary file name.
            self.args.update({"sdkconfig-rename": f.name})
            # The content of our `rename` file is simple: replace `CONFIG_NAMED_OPTION` by `CONFIG_RENAMED_OPTION`
            f.write(
                textwrap.dedent(
                    """
            CONFIG_NAMED_OPTION             CONFIG_RENAMED_OPTION
            """
                )
            )

    def testRenamedOptionDisabled(self):
        # Invoke the unit test, specify that the final `sdkconfig` generated must contain the string:
        # "# CONFIG_RENAMED_OPTION is not set"
        self.invoke_and_test(self.input, "# CONFIG_RENAMED_OPTION is not set")


class HeaderTestCase(KconfgenBaseTestCase):
    @classmethod
    def setUpClass(self):
        super(HeaderTestCase, self).setUpClass()
        self.args.update({"output": "header"})

    def testStringEscape(self):
        self.invoke_and_test(
            """
        config PASSWORD
            string "password"
            default "\\\\~!@#$%^&*()\\\""
        """,
            '#define CONFIG_PASSWORD "\\\\~!@#$%^&*()\\""',
        )

    def testHexPrefix(self):
        self.invoke_and_test(HEXPREFIX_KCONFIG, "#define CONFIG_HEX_NOPREFIX 0x33")
        self.invoke_and_test(HEXPREFIX_KCONFIG, "#define CONFIG_HEX_PREFIX 0x77")


class DocsTestCase(KconfgenBaseTestCase):
    @classmethod
    def setUpClass(self):
        super(DocsTestCase, self).setUpClass()
        self.args.update({"output": "docs", "env": "IDF_TARGET=esp32"})

    def testMultipleRangesWithNegation(self):
        out = self.invoke_and_test(
            """
        config IDF_TARGET
            string "IDF target"
            default "esp32"

        config IDF_TARGET_ESP32
            bool
            default "y" if IDF_TARGET="esp32"

        config SOME_SETTING
            int "setting for the chip"
            range 0 10 if IDF_TARGET_ESP32
            range 0 100 if !IDF_TARGET_ESP32
        """,
            re.compile(r"Range:\n\s+- from 0 to 10"),
            "regex",
        )
        assert "- from 0 to 100" not in out

    def testChoice(self):
        self.invoke_and_test(
            """
        menu "TEST"
            choice TYPES
                prompt "types"
                default TYPES_OP2
                help
                    Description of TYPES

                config TYPES_OP1
                    bool "option 1"
                config TYPES_OP2
                    bool "option 2"
            endchoice
        endmenu
        """,
            """
        TEST
        ----

        Contains:

        - :ref:`CONFIG_TYPES`

        .. _CONFIG_TYPES:

        CONFIG_TYPES
        ^^^^^^^^^^^^

            types

            :emphasis:`Found in:` :ref:`test`

            Description of TYPES

            Available options:

                  .. _CONFIG_TYPES_OP1:

                - option 1             (CONFIG_TYPES_OP1)

                  .. _CONFIG_TYPES_OP2:

                - option 2             (CONFIG_TYPES_OP2)

        """,
        )  # this is more readable than regex


class DefaultsTestCase(KconfgenBaseTestCase):
    @classmethod
    def setUpClass(self):
        super(DefaultsTestCase, self).setUpClass()
        self.args.update({"output": "savedefconfig"})
        self.input = """
        menu "Label"
            config IDF_TARGET
                string "IDF target"
                default "esp32"

            menu "This is a menu label"
                config TEST2
                    bool "test"
                    default "y"
            endmenu

            menu "Label"
                config TEST
                    bool "test"
                    default "y"
                    comment "This is a comment for TEST"
            endmenu
        endmenu
        """

    def setUp(self):
        super(DefaultsTestCase, self).setUp()
        with tempfile.NamedTemporaryFile(
            mode="w+", prefix="test_kconfgen_", delete=False
        ) as f:
            self.addCleanup(os.remove, f.name)
            self.args.update(
                {"config": f.name}
            )  # this is input in contrast with {'output': 'config'}
            f.write(
                textwrap.dedent(
                    """
                    CONFIG_TEST=n
                    CONFIG_TEST2=n
                    """
                )
            )

    def testSaveDefault(self):
        # Make sure that setting bool to false is represented as assignment and not as comment "CONFIG_TEST is not set"
        self.invoke_and_test(self.input, "CONFIG_TEST=n")
        self.invoke_and_test(self.input, "# CONFIG_TEST is not set", "not in")

    def testSaveDefaultWithLabels(self):
        # get min config without labels and with labes, remove labes and compare the results

        # without labels
        self.invoke_and_test(self.input, "# This is a menu label", "not in")
        with open(self.output_file, "r") as f:
            without_labels = f.readlines()
        # set min config labels
        os.environ["ESP_IDF_KCONFIG_MIN_LABELS"] = "1"
        # add cleanup to remove the env variable after test
        self.addCleanup(os.environ.pop, "ESP_IDF_KCONFIG_MIN_LABELS", None)
        # get result with labels
        self.invoke_and_test(self.input, "# This is a menu label")
        with open(self.output_file, "r") as f:
            with_labels = f.readlines()
        # verify that comments are not printed out as labels
        self.assertNotIn("# This is a comment for TEST\n", with_labels)
        # verify that "Label" is twice in the min config
        cnt = 0
        for line in with_labels:
            if line == "# Label\n":
                cnt += 1
        self.assertEqual(cnt, 2)
        # remove labels and empty lines; ignore first three lines that contain header
        with_labels_strip = with_labels[:3] + list(
            filter(lambda x: not x.startswith("#") and not x == "\n", with_labels[3:])
        )
        self.assertEqual(without_labels, with_labels_strip)


# Used by multiple testHexPrefix() test cases to verify correct hex output for each format
HEXPREFIX_KCONFIG = """
config HEX_NOPREFIX
hex "Hex Item default no prefix"
default 33

config HEX_PREFIX
hex "Hex Item default prefix"
default 0x77
"""

if __name__ == "__main__":
    unittest.main()



print("ESP-IDF Kconfig tool")
msg = "Please select a tool to run with command:"
print(
    f"{msg}"
    f"\n{' '*int(len(msg)/2)}"
    f"Kconfig file checker. {' '*8} (python -m kconfcheck)"
    f"\n{' '*int(len(msg)/2)}"
    "Run JSON configuration server. (idf.py confserver or python -m kconfserver)"
    f"\n{' '*int(len(msg)/2)}"
    f"Config Generation Tool. {' '*6} (python -m kconfgen)"
)



#!/usr/bin/env python
import os
import unittest

import kconfiglib

from esp_idf_kconfig import gen_kconfig_doc


class ConfigTargetVisibilityTestCase(unittest.TestCase):
    def setUp(self):
        self.target = os.environ["IDF_TARGET"]
        self.config = kconfiglib.Kconfig("Kconfig")
        self.v = gen_kconfig_doc.ConfigTargetVisibility(self.config, self.target)

    def _get_config(self, name):
        sym = self.config.syms.get(name)
        if sym and len(sym.nodes) > 0:
            return sym.nodes[0]
        choice = self.config.named_choices.get(name)
        if choice:
            return choice.nodes[0]
        raise RuntimeError("Unimplemented {}".format(name))

    def visible(self, config_name):
        self.assertTrue(self.v.visible(self._get_config(config_name)))

    def invisible(self, config_name):
        self.assertFalse(self.v.visible(self._get_config(config_name)))


class ConfigTargetVisibilityChipA(ConfigTargetVisibilityTestCase):
    @classmethod
    def setUpClass(cls):
        os.environ["IDF_TARGET"] = "chipa"

    def test_config_visibility(self):
        self.invisible("IDF_TARGET")
        self.invisible("IDF_TARGET_CHIPA")
        self.visible("ALWAYS_VISIBLE")
        self.visible("ALWAYS_VISIBLE_CHOICE")
        self.visible("CONFIG_FOR_CHIPA")
        self.invisible("CONFIG_FOR_CHIPB")
        self.visible("CHOICE_FOR_CHIPA")
        self.invisible("CHOICE_FOR_CHIPB")
        self.visible("EXT_CONFIG1_FOR_CHIPA_MENU")
        self.visible("EXT_CONFIG2_FOR_CHIPA_MENU")
        self.visible("EXT_CONFIG3_FOR_CHIPA")
        self.invisible("EXT_CONFIG1_FOR_CHIPB_MENU")
        self.invisible("EXT_CONFIG2_FOR_CHIPB_MENU")
        self.invisible("EXT_CONFIG3_FOR_CHIPB")
        self.visible("EXT_CONFIG4")
        self.visible("DEEP_DEPENDENT_CONFIG")
        self.visible("DEEP_DEPENDENT_CONFIG_INV")
        self.visible("DEEP_DEPENDENT_CHOICE")
        self.invisible("INVISIBLE1")
        self.visible("VISIBLE1")
        self.visible("CONFIG_FOR_CHIPA_DEPENDS_VAR1")
        self.visible("CONFIG_FOR_CHIPA_DEPENDS_VAR2")
        self.visible("CONFIG_FOR_CHIPA_DEPENDS_VAR3")
        self.visible("CONFIG_DEPENDS_ENV_VAR1")
        self.visible("CONFIG_DEPENDS_ENV_VAR2")
        self.visible("CHIPA_VERSION")
        self.invisible("CHIPA_REV_MIN")
        self.visible("CHIPA_FEATURE_FROM_V1")
        self.visible("CHIPA_FEATURE_FROM_V3")


class ConfigTargetVisibilityChipB(ConfigTargetVisibilityTestCase):
    @classmethod
    def setUpClass(cls):
        os.environ["IDF_TARGET"] = "chipb"

    def test_config_visibility(self):
        self.invisible("IDF_TARGET")
        self.invisible("IDF_TARGET_CHIPA")
        self.visible("ALWAYS_VISIBLE")
        self.visible("ALWAYS_VISIBLE_CHOICE")
        self.invisible("CONFIG_FOR_CHIPA")
        self.visible("CONFIG_FOR_CHIPB")
        self.invisible("CHOICE_FOR_CHIPA")
        self.visible("CHOICE_FOR_CHIPB")
        self.invisible("EXT_CONFIG1_FOR_CHIPA_MENU")
        self.invisible("EXT_CONFIG2_FOR_CHIPA_MENU")
        self.invisible("EXT_CONFIG3_FOR_CHIPA")
        self.visible("EXT_CONFIG1_FOR_CHIPB_MENU")
        self.visible("EXT_CONFIG2_FOR_CHIPB_MENU")
        self.visible("EXT_CONFIG3_FOR_CHIPB")
        self.visible("EXT_CONFIG4")
        self.invisible("DEEP_DEPENDENT_CONFIG")
        self.visible("DEEP_DEPENDENT_CONFIG_INV")
        self.invisible("DEEP_DEPENDENT_CHOICE")
        self.invisible("INVISIBLE1")
        self.visible("VISIBLE1")
        self.invisible("CONFIG_FOR_CHIPA_DEPENDS_VAR1")
        self.invisible("CONFIG_FOR_CHIPA_DEPENDS_VAR2")
        self.invisible("CONFIG_FOR_CHIPA_DEPENDS_VAR3")
        self.visible("CONFIG_DEPENDS_ENV_VAR1")
        self.visible("CONFIG_DEPENDS_ENV_VAR2")
        self.invisible("CHIPA_VERSION")
        self.invisible("CHIPA_REV_MIN")
        self.invisible("CHIPA_FEATURE_FROM_V1")
        self.invisible("CHIPA_FEATURE_FROM_V3")


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
from __future__ import print_function

import argparse
import json
import os
import re
import tempfile

import pexpect

# Each protocol version to be tested needs a 'testcases_vX.txt' file
PROTOCOL_VERSIONS = [1, 2]


def parse_testcases(version):
    with open("testcases_v%d.txt" % version, "r") as f:
        cases = [line for line in f.readlines() if len(line.strip()) > 0]
    # Each 3 lines in the file should be formatted as:
    # * Description of the test change
    # * JSON "changes" to send to the server
    # * Result JSON to expect back from the server
    if len(cases) % 3 != 0:
        print(
            "Warning: testcases.txt has wrong number of non-empty lines (%d). Should be 3 lines per test case, always."
            % len(cases)
        )

    for i in range(0, len(cases), 3):
        desc = cases[i]
        send = cases[i + 1]
        expect = cases[i + 2]
        if not desc.startswith("* "):
            raise RuntimeError(
                "Unexpected description at line %d: '%s'" % (i + 1, desc)
            )
        if not send.startswith("> "):
            raise RuntimeError("Unexpected send at line %d: '%s'" % (i + 2, send))
        if not expect.startswith("< "):
            raise RuntimeError("Unexpected expect at line %d: '%s'" % (i + 3, expect))
        desc = desc[2:]
        send = json.loads(send[2:])
        expect = json.loads(expect[2:])
        yield (desc, send, expect)


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "--logfile",
        type=argparse.FileType("w"),
        help="Optional session log of the interactions with kconfserver.py",
    )
    args = parser.parse_args()

    try:
        # set up temporary file to use as sdkconfig copy
        with tempfile.NamedTemporaryFile(mode="w", delete=False) as temp_sdkconfig:
            temp_sdkconfig_path = os.path.join(
                tempfile.gettempdir(), temp_sdkconfig.name
            )
            with open("sdkconfig") as orig:
                temp_sdkconfig.write(orig.read())

        with tempfile.NamedTemporaryFile(delete=False) as f:
            temp_kconfigs_source_file = os.path.join(tempfile.gettempdir(), f.name)

        with tempfile.NamedTemporaryFile(delete=False) as f:
            temp_kconfig_projbuilds_source_file = os.path.join(
                tempfile.gettempdir(), f.name
            )

        cmdline = """python -m kconfserver --env "COMPONENT_KCONFIGS_SOURCE_FILE=%s" \
                                           --env "COMPONENT_KCONFIGS_PROJBUILD_SOURCE_FILE=%s" \
                                           --env "COMPONENT_KCONFIGS=" \
                                           --env "COMPONENT_KCONFIGS_PROJBUILD=" \
                                           --kconfig Kconfig \
                                           --config %s \
                  """ % (
            temp_kconfigs_source_file,
            temp_kconfig_projbuilds_source_file,
            temp_sdkconfig_path,
        )

        cmdline = re.sub(r" +", " ", cmdline)

        print("Running: %s" % cmdline)
        p = pexpect.spawn(
            cmdline,
            timeout=30,
            logfile=args.logfile,
            echo=False,
            use_poll=True,
            maxread=1,
        )

        p.expect("Server running.+\r\n")
        initial = expect_json(p)
        print("Initial: %s" % initial)

        for version in PROTOCOL_VERSIONS:
            test_protocol_version(p, version)

        test_load_save(p, temp_sdkconfig_path)

        test_invalid_json(p)

        print("Done. All passed.")

    finally:
        try:
            os.remove(temp_sdkconfig_path)
            os.remove(temp_kconfigs_source_file)
            os.remove(temp_kconfig_projbuilds_source_file)
        except OSError:
            pass


def expect_json(p):
    # run p.expect() to expect a json object back, and return it as parsed JSON
    p.expect("{.+}\r\n")
    result = p.match.group(0).strip().decode()
    print("Read raw data from server: %s" % result)
    return json.loads(result)


def send_request(p, req):
    req = json.dumps(req)
    print("Sending: %s" % (req))
    p.send("%s\n" % req)
    readback = expect_json(p)
    print("Read back: %s" % (json.dumps(readback)))
    return readback


def test_protocol_version(p, version):
    print("*****")
    print("Testing version %d..." % version)

    # reload the config from the sdkconfig file
    req = {"version": version, "load": None}
    readback = send_request(p, req)
    print("Reset response: %s" % (json.dumps(readback)))

    # run through each test case
    cases = parse_testcases(version)
    for (desc, send, expected) in cases:
        print(desc)
        req = {"version": version, "set": send}
        readback = send_request(p, req)
        if readback.get("version", None) != version:
            raise RuntimeError('Expected {"version" : %d} in response' % version)
        for expect_key in expected.keys():
            read_vals = readback[expect_key]
            exp_vals = expected[expect_key]
            if read_vals != exp_vals:
                expect_diff = dict(
                    (k, v)
                    for (k, v) in exp_vals.items()
                    if k not in read_vals or v != read_vals[k]
                )
                raise RuntimeError(
                    "Test failed! Was expecting %s: %s"
                    % (expect_key, json.dumps(expect_diff))
                )
        print("OK")
    print("Version %d OK" % version)


def test_load_save(p, temp_sdkconfig_path):
    print("Testing load/save...")
    before = os.stat(temp_sdkconfig_path).st_mtime
    save_result = send_request(p, {"version": 2, "save": temp_sdkconfig_path})
    print("Save result: %s" % (json.dumps(save_result)))
    assert "error" not in save_result
    assert len(save_result["values"]) == 0  # nothing changes when we save
    assert len(save_result["ranges"]) == 0
    after = os.stat(temp_sdkconfig_path).st_mtime
    assert after > before  # something got written to disk

    # Do a V2 load
    load_result = send_request(p, {"version": 2, "load": temp_sdkconfig_path})
    print("V2 Load result: %s" % (json.dumps(load_result)))
    assert "error" not in load_result
    assert (
        len(load_result["values"]) == 0
    )  # in V2, loading same file should return no config items
    assert len(load_result["ranges"]) == 0

    # Do a V1 load
    load_result = send_request(p, {"version": 1, "load": temp_sdkconfig_path})
    print("V1 Load result: %s" % (json.dumps(load_result)))
    assert "error" not in load_result
    assert (
        len(load_result["values"]) > 0
    )  # in V1, loading same file should return all config items
    assert len(load_result["ranges"]) > 0


def test_invalid_json(p):
    print("Testing invalid JSON formatting...")

    bad_escaping = r'{ "version" : 2, "load" : "c:\some\path\not\escaped\as\json" }'
    p.send("%s\n" % bad_escaping)
    readback = expect_json(p)
    print(readback)
    assert "json" in readback["error"][0].lower()

    not_really_json = "Hello world!!"
    p.send("%s\n" % not_really_json)
    readback = expect_json(p)
    print(readback)
    assert "json" in readback["error"][0].lower()


if __name__ == "__main__":
    main()



#!/usr/bin/env python
#
# SPDX-FileCopyrightText: 2018-2023 Espressif Systems (Shanghai) CO LTD
# SPDX-License-Identifier: Apache-2.0

import unittest

from kconfcheck.core import (
    CONFIG_NAME_MAX_LENGTH,
    IndentAndNameChecker,
    InputError,
    LineRuleChecker,
    SourceChecker,
)


class ApplyLine(object):
    def apply_line(self, string):
        self.checker.process_line(string + "\n", 0)

    def expect_error(self, string, expect, cleanup=None):
        try:
            with self.assertRaises(InputError) as cm:
                self.apply_line(string)
            if expect:
                self.assertEqual(cm.exception.suggested_line, expect + "\n")
        finally:
            if cleanup:
                # cleanup of the previous failure
                self.apply_line(cleanup)

    def expt_success(self, string):
        self.apply_line(string)


class TestLineRuleChecker(unittest.TestCase, ApplyLine):
    def setUp(self):
        self.checker = LineRuleChecker("Kconfig")

    def tearDown(self):
        pass

    def test_tabulators(self):
        self.expect_error("\ttest", expect="    test")
        self.expect_error("\t    test", expect="        test")
        self.expect_error("   \ttest", expect="       test")
        self.expect_error("     \t     test", expect="              test")
        self.expt_success("         test")
        self.expt_success("test")

    def test_trailing_whitespaces(self):
        self.expect_error(" ", expect="")
        self.expect_error("     ", expect="")
        self.expect_error("test ", expect="test")
        self.expt_success("test")
        self.expt_success("")

    def test_line_length(self):
        self.expect_error("x" * 120, expect=None)
        self.expt_success("x" * 119)
        self.expt_success("")


class TestSourceChecker(unittest.TestCase, ApplyLine):
    def setUp(self):
        self.checker = SourceChecker("Kconfig")

    def tearDown(self):
        pass

    def test_source_file_name(self):
        self.expect_error(
            'source "notKconfig.test"', expect='source "Kconfig.notKconfig.test"'
        )
        self.expect_error('source "Kconfig"', expect='source "Kconfig.Kconfig"')
        self.expt_success('source "Kconfig.in"')
        self.expt_success('source "/tmp/Kconfig.test"')
        self.expt_success('source "/tmp/Kconfig.in"')
        self.expect_error('source"Kconfig.in"', expect='source "Kconfig.in"')
        self.expt_success('source "/tmp/Kconfig.in"  # comment')


class TestIndentAndNameChecker(unittest.TestCase, ApplyLine):
    def setUp(self):
        self.checker = IndentAndNameChecker("Kconfig")
        self.checker.min_prefix_length = 4

    def tearDown(self):
        self.checker.__exit__("Kconfig", None, None)


class TestIndent(TestIndentAndNameChecker):
    def setUp(self):
        super(TestIndent, self).setUp()
        self.checker.min_prefix_length = 0  # prefixes are ignored in this test case

    def test_indent_characters(self):
        self.expt_success('menu "test"')
        self.expect_error(" test", expect="    test")
        self.expect_error("  test", expect="    test")
        self.expect_error("   test", expect="    test")
        self.expect_error("     test", expect="    test")
        self.expt_success("    test")
        self.expt_success("    test2")
        self.expt_success("    config")
        self.expect_error("    default", expect="        default")
        self.expt_success("        help")
        self.expect_error("         text", expect="            text")
        self.expt_success("            help text")
        self.expt_success("    menu")
        self.expt_success("    endmenu")
        self.expect_error(
            "         choice", expect="    choice", cleanup="    endchoice"
        )
        self.expect_error("       choice", expect="    choice", cleanup="    endchoice")
        self.expt_success("    choice")
        self.expt_success("    endchoice")
        self.expt_success("    config")
        self.expt_success("endmenu")

    def test_help_content(self):
        self.expt_success('menu "test"')
        self.expt_success("    config")
        self.expt_success("        help")
        self.expt_success("            description")
        self.expt_success("            config keyword in the help")
        self.expt_success("            menu keyword in the help")
        self.expt_success("            menuconfig keyword in the help")
        self.expt_success("            endmenu keyword in the help")
        self.expt_success("            endmenu keyword in the help")
        self.expt_success("")  # newline in help
        self.expt_success("            endmenu keyword in the help")
        self.expect_error(
            '          menu "real menu with wrong indent"',
            expect='    menu "real menu with wrong indent"',
            cleanup="    endmenu",
        )
        self.expt_success("endmenu")

    def test_mainmenu(self):
        self.expt_success('mainmenu "test"')
        self.expect_error("test", expect="    test")
        self.expt_success("    not_a_keyword")
        self.expt_success("    config")
        self.expt_success("    menuconfig")
        self.expect_error("test", expect="        test")
        self.expect_error("   test", expect="        test")
        self.expt_success("    menu")
        self.expt_success("    endmenu")

    def test_ifendif(self):
        self.expt_success('menu "test"')
        self.expt_success("    config")
        self.expt_success("        help")
        self.expect_error("        if", expect="    if", cleanup="    endif")
        self.expt_success("    if")
        self.expect_error("    config", expect="        config")
        self.expt_success("        config")
        self.expt_success("            help")
        self.expt_success("    endif")
        self.expt_success("    config")
        self.expt_success("endmenu")

    def test_config_without_menu(self):
        self.expt_success("menuconfig")
        self.expt_success("    help")
        self.expt_success("        text")
        self.expt_success("")
        self.expt_success("        text")
        self.expt_success("config")
        self.expt_success("    help")

    def test_source_after_config(self):
        self.expt_success("menuconfig")
        self.expt_success("    help")
        self.expt_success("        text")
        self.expect_error("    source", expect="source")
        self.expt_success('source "Kconfig.in"')

    def test_comment_after_config(self):
        self.expt_success("menuconfig")
        self.expt_success("    # comment")
        self.expt_success("    help")
        self.expt_success("        text")
        self.expt_success('        # second not realcomment"')


class TestName(TestIndentAndNameChecker):
    def setUp(self):
        super(TestName, self).setUp()
        self.checker.min_prefix_length = 0  # prefixes are ignored in this test case

    def test_name_length(self):
        max_length = CONFIG_NAME_MAX_LENGTH
        too_long = max_length + 1
        self.expt_success('menu "test"')
        self.expt_success("    config ABC")
        self.expt_success("    config " + ("X" * max_length))
        self.expect_error("    config " + ("X" * too_long), expect=None)
        self.expt_success("    menuconfig " + ("X" * max_length))
        self.expect_error("    menuconfig " + ("X" * too_long), expect=None)
        self.expt_success("    choice " + ("X" * max_length))
        self.expect_error("    choice " + ("X" * too_long), expect=None)
        self.expt_success("endmenu")


class TestPrefix(TestIndentAndNameChecker):
    def test_prefix_len(self):
        self.expt_success('menu "test"')
        self.expt_success("    config ABC_1")
        self.expt_success("    config ABC_2")
        self.expt_success("    config ABC_DEBUG")
        self.expt_success("    config ABC_ANOTHER")
        self.expt_success("endmenu")
        self.expt_success('menu "test2"')
        self.expt_success("    config A")
        self.expt_success("    config B")
        self.expect_error("endmenu", expect=None)

    def test_choices(self):
        self.expt_success('menu "test"')
        self.expt_success("    choice ASSERTION_LEVEL")
        self.expt_success("        config ASSERTION_DEBUG")
        self.expt_success("        config ASSERTION_RELEASE")
        self.expt_success("        menuconfig ASSERTION_XY")
        self.expt_success("    endchoice")
        self.expt_success("    choice DEBUG")
        self.expt_success("        config DE_1")
        self.expt_success("        config DE_2")
        self.expect_error("    endchoice", expect=None)
        self.expect_error("endmenu", expect=None)

    def test_nested_menu(self):
        self.expt_success('menu "test"')
        self.expt_success("    config DOESNT_MATTER")
        self.expt_success('    menu "inner menu"')
        self.expt_success("        config MENUOP_1")
        self.expt_success("        config MENUOP_2")
        self.expt_success("        config MENUOP_3")
        self.expt_success("    endmenu")
        self.expt_success("endmenu")

    def test_nested_ifendif(self):
        self.expt_success('menu "test"')
        self.expt_success("    config MENUOP_1")
        self.expt_success("    if MENUOP_1")
        self.expt_success("        config MENUOP_2")
        self.expt_success("    endif")
        self.expt_success("endmenu")


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
from __future__ import unicode_literals

import io
import os
import sys
import unittest

import kconfiglib

from esp_idf_kconfig import gen_kconfig_doc


class TestDocOutput(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        os.environ["IDF_TARGET"] = "chipa"
        cls.target = os.environ["IDF_TARGET"]
        cls.config = kconfiglib.Kconfig("Kconfig")
        cls.visibility = gen_kconfig_doc.ConfigTargetVisibility(cls.config, cls.target)

    def get_config(self, name):
        sym = self.config.syms.get(name)
        if sym:
            return sym.nodes[0]
        choice = self.config.named_choices.get(name)
        if choice:
            return choice.nodes[0]
        raise RuntimeError("Unimplemented {}".format(name))

    def get_doc_out(self, config_name):
        with io.StringIO() if sys.version_info.major == 3 else io.BytesIO() as output:
            gen_kconfig_doc.write_menu_item(
                output, self.get_config(config_name), self.visibility
            )
            output.seek(0)
            return output.read()

    def test_simple_default(self):
        s = self.get_doc_out("EXT_CONFIG3_FOR_CHIPA_MENU")
        self.assertIn("- 5", s)

    def test_multiple_defaults(self):
        s = self.get_doc_out("CHIPA_OPTION")
        self.assertNotIn("- 5", s)
        self.assertIn("- 4 if CHIPA_VERSION = 2", s)
        self.assertNotIn("- 9", s)

    def test_string_default(self):
        s = self.get_doc_out("COMPILER")
        self.assertIn("- ca", s)
        self.assertNotIn("- cb", s)

    def test_bool_default(self):
        s = self.get_doc_out("BOOL_OPTION")
        self.assertIn("- Yes", s)

    def test_bool_default_dependency(self):
        s = self.get_doc_out("BOOL_OPTION2")
        self.assertIn("- Yes", s)

    def test_hex_default(self):
        s = self.get_doc_out("HEX_OPTION")
        self.assertIn('- "0xce"', s)
        self.assertNotIn('- "0xff"', s)

    def test_hex_range(self):
        s = self.get_doc_out("HEX_OPTION")
        self.assertIn("- from 0xf to 0xce", s)
        self.assertNotIn("- from 0xfe", s)

    def test_int_range(self):
        s = self.get_doc_out("INT_OPTION")
        self.assertIn("- from 1 to 10", s)
        self.assertNotIn("- from 100", s)

    def test_choice(self):
        s = self.get_doc_out("CHOICE_FOR_CHIPA")
        self.assertIn("Available options:", s)
        self.assertRegex(s, r"- op1\s+\(CONFIG_CHOICE_FOR_CHIPA_OP1\)")
        self.assertRegex(s, r"- op2\s+\(CONFIG_CHOICE_FOR_CHIPA_OP2\)")

        s = self.get_doc_out("OPT_DEPENDENT_ON_CHOICE_OP2")
        self.assertIn(
            "- Yes (enabled) if :ref:`CONFIG_CHOICE_FOR_CHIPA_OP2<CONFIG_CHOICE_FOR_CHIPA_OP2>`",
            s,
        )


if __name__ == "__main__":
    unittest.main()



import sys

from .core import main


class FatalError(RuntimeError):
    """
    Class for runtime errors (not caused by bugs but by user input).
    """

    pass


if __name__ == "__main__":
    try:
        main()
    except FatalError as e:
        print("A fatal error occurred: %s" % e)
        sys.exit(2)
